@@ -120,12 +120,12 @@ sub Run {
     if ( !IsHashRefWithData($Webservice) ) {
         $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
-            Message =>
+            Message  =>
                 "Could not load web service configuration for web service $Param{WebserviceID}",
         );
 
         return {
-            Success => 0,
+            Success      => 0,
             ErrorMessage =>
                 "Could not load web service configuration for web service $Param{WebserviceID}",
         };
@@ -197,7 +197,9 @@ sub Run {
     );
 
     my $FunctionResult = $InvokerObject->PrepareRequest(
-        Data => $Param{Data},
+        Webservice  => $Webservice,
+        InvokerName => $Param{Invoker},
+        Data        => $Param{Data},
     );
 
     if ( !$FunctionResult->{Success} ) {
@@ -248,7 +250,7 @@ sub Run {
             DebuggerObject => $DebuggerObject,
             Invoker        => $Param{Invoker},
             InvokerType    => $RequesterConfig->{Invoker}->{ $Param{Invoker} }->{Type},
-            MappingConfig =>
+            MappingConfig  =>
                 $RequesterConfig->{Invoker}->{ $Param{Invoker} }->{MappingOutbound},
         );
 
@@ -308,8 +310,10 @@ sub Run {
 
     # Read request content.
     $FunctionResult = $TransportObject->RequesterPerformRequest(
-        Operation => $Param{Invoker},
-        Data      => $DataOut,
+        Webservice => $Webservice,
+        Sort       => $RequesterConfig->{Invoker}->{ $Param{Invoker} }->{SortOutbound},
+        Operation  => $Param{Invoker},
+        Data       => $DataOut,
     );
 
     my $IsAsynchronousCall = $Param{Asynchronous} ? 1 : 0;
@@ -351,10 +355,10 @@ sub Run {
     # Extend the data include payload.
     $DataInclude{RequesterResponseInput} = $FunctionResult->{Data};
 
-    my $DataIn      = $FunctionResult->{Data};
-    my $SizeExeeded = $FunctionResult->{SizeExeeded} || 0;
+    my $DataIn       = $FunctionResult->{Data};
+    my $SizeExceeded = $FunctionResult->{SizeExceeded} || 0;
 
-    if ($SizeExeeded) {
+    if ($SizeExceeded) {
         $DebuggerObject->Debug(
             Summary => "Incoming data before mapping was too large for logging",
             Data => 'See SysConfig option GenericInterface::Operation::ResponseLoggingMaxSize to change the maximum.',
@@ -378,7 +382,7 @@ sub Run {
             DebuggerObject => $DebuggerObject,
             Invoker        => $Param{Invoker},
             InvokerType    => $RequesterConfig->{Invoker}->{ $Param{Invoker} }->{Type},
-            MappingConfig =>
+            MappingConfig  =>
                 $RequesterConfig->{Invoker}->{ $Param{Invoker} }->{MappingInbound},
         );
 
@@ -416,10 +420,10 @@ sub Run {
 
         $DataIn = $FunctionResult->{Data};
 
-        if ($SizeExeeded) {
+        if ($SizeExceeded) {
             $DebuggerObject->Debug(
                 Summary => "Incoming data after mapping was too large for logging",
-                Data =>
+                Data    =>
                     'See SysConfig option GenericInterface::Operation::ResponseLoggingMaxSize to change the maximum.',
             );
         }
