@@ -1,6 +1,6 @@
 # --
 # Copyright (C) 2001-2021 OTRS AG, https://otrs.com/
-# Copyright (C) 2021-2022 Znuny GmbH, https://znuny.org/
+# Copyright (C) 2021 Znuny GmbH, https://znuny.org/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -15,7 +15,7 @@ use warnings;
 use List::Util qw( first );
 
 use Kernel::System::VariableCheck qw(:all);
-use Kernel::Language qw(Translatable);
+use Kernel::Language              qw(Translatable);
 
 our $ObjectManagerDisabled = 1;
 
@@ -113,11 +113,11 @@ sub OverviewScreen {
     my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
 
     # Get Params
-    $Param{SearchPageShown} = $ConfigObject->Get('Stats::SearchPageShown') || 50;
-    $Param{SearchLimit}     = $ConfigObject->Get('Stats::SearchLimit')     || 1000;
-    $Param{OrderBy}   = $ParamObject->GetParam( Param => 'OrderBy' )   || 'ID';
-    $Param{Direction} = $ParamObject->GetParam( Param => 'Direction' ) || 'ASC';
-    $Param{StartHit} = int( $ParamObject->GetParam( Param => 'StartHit' ) || 1 );
+    $Param{SearchPageShown} = $ConfigObject->Get('Stats::SearchPageShown')   || 50;
+    $Param{SearchLimit}     = $ConfigObject->Get('Stats::SearchLimit')       || 1000;
+    $Param{OrderBy}         = $ParamObject->GetParam( Param => 'OrderBy' )   || 'ID';
+    $Param{Direction}       = $ParamObject->GetParam( Param => 'Direction' ) || 'ASC';
+    $Param{StartHit}        = int( $ParamObject->GetParam( Param => 'StartHit' ) || 1 );
 
     # get all Stats from the db
     my $Result = $Kernel::OM->Get('Kernel::System::Stats')->GetStatsList(
@@ -317,11 +317,19 @@ sub DeleteAction {
 
     # challenge token check for write action
     $LayoutObject->ChallengeTokenCheck();
-    $Kernel::OM->Get('Kernel::System::Stats')->StatsDelete(
+    my $Success = $Kernel::OM->Get('Kernel::System::Stats')->StatsDelete(
         StatID => $StatID,
         UserID => $Self->{UserID},
     );
-    return $LayoutObject->Redirect( OP => "Action=AgentStatistics;Subaction=Overview" );
+
+    return $LayoutObject->Attachment(
+        ContentType => 'text/html',
+        Content     => ($Success) ? 1 : 0,
+        Type        => 'inline',
+        NoCache     => 1,
+    );
+
+    #     return $LayoutObject->Redirect( OP => "Action=AgentStatistics;Subaction=Overview" );
 }
 
 sub EditScreen {
@@ -729,8 +737,8 @@ sub ViewScreen {
         UserID => $Self->{UserID},
     );
     if ( !IsHashRefWithData( $StatsList->{$StatID} ) ) {
-        return $LayoutObject->ErrorScreen(
-            Message => Translatable('Could not load stat.'),
+        return $LayoutObject->NoPermission(
+            Message => Translatable("This stat does not exist, or you don't have permissions to access it."),
         );
     }
 
